#ifndef __JOBWIDGET_H
#define __JOBWIDGET_H

#include <QTreeWidget>
#include <QPointer>
#include <QHash>
#include "fly-admin-printer_export.h"

class QAction;
class QMenu;
class QTimer;
class SecurityJobAction;

namespace Cups
{
class Connection;
class Job;
}

class FLY_ADMIN_PRINTER_EXPORT JobWidget: public QTreeWidget
{
    Q_OBJECT
public:
    enum Action {
        HoldReleaseAction = 0,
        CancelAction,
        RestartAction,
        AuthAction,
        MoveAction,
        PurgeAction,
        ShowCompletedAction,
        ShowCanceledAction,
        ShowAbortedAction,
        ShowMyJobsOnlyAction,
        ShowAttributesAction
    };

    explicit JobWidget(QWidget *parent = nullptr);
    ~JobWidget();

    void load(Cups::Connection *connection, const QString &printer = QString(), bool isClass = false);
    QAction *action(Action id) const;
    QList<QAction *> extraActions() const;

public slots:
    void currentItemChanged(QTreeWidgetItem*it, QTreeWidgetItem*);
    void reload();

protected:
    void contextMenuEvent(QContextMenuEvent *event) override;
    void keyPressEvent(QKeyEvent *event) override;

protected slots:
    void onJobState(const QString &text, const QString &uri, const QString &name);

private slots:
    void cancelJob();
    void purgeJob();
    void holdReleaseJob();
    void restartJob();
    void authenticateJob();
    void moveJob();
    void showAttributes();

signals:
    void jobsStateChanged();

private:
    void connectToDBus();
    void disconnectFromDBus();
    void configureItem(QTreeWidgetItem *item, const Cups::Job &job, const QList <QVariantMap> &dests, bool update);
    QTimer *m_refreshTimer;
    QList<SecurityJobAction*> m_extraActions;
    Cups::Connection *m_connection = nullptr;
    QString m_printer;
    QMenu *m_menu;
    QHash<Action, QAction *> m_actions;
    bool m_isClass = false;
};

#endif
